@@ -862,5 +862,13 @@ void SList<T>::sort(int from /* = 0 */, int to /* = - 1 */, void *data /* = NULL
 /******************* expression list moved here ***************/
 typedef PList<Expression *> ExprList;
 
+/******************* sorted string list ***********************/
+class SortedStringList: public SList<Str*> {
+ public:
+  SortedStringList() : SList<Str*>(LIST_SIZE_MEDIUM) {};
+  virtual int compare(int, int, void* data);
+  int findIdx(Str & val);
+};
+
 #endif //ifndef DataStrHIncl
 
