@@ -125,19 +125,6 @@ DataLineItem::~DataLineItem()
         cdelete(_tree);
 }
 
-/************************************************************
-
-OutputDocumentList
-
-************************************************************/
-
-eFlag OutputDocumentList::finish(Sit S)
-{
-  for (int i = 0; i < number(); i++)
-    E( (*this)[i] -> finish(S) );
-  return OK;
-}
-
 /*****************************************************************
 
   DataLinesList
@@ -274,18 +261,19 @@ These deletes are now subsumed in "datalines.freeall()":
 #endif
 
     outputDocuments.freeall(FALSE);
+    outputDocumentURIs.freeall(FALSE);
 
     freeNonArgDatalines();
     if (Sp && !Sp -> isError())
-    {
+      {
         assert(modes.isEmpty());
         assert(outputters_.isEmpty());
-    }
+      }
     else
-    {
+      {
         modes.freeall(FALSE);
         outputters_.freeall(FALSE);
-    };
+      };
     if (Sp)
 	Sp -> clear();
     // hope this works:
@@ -313,6 +301,8 @@ eFlag Processor::open(Sit S, const char *sheetURI, const char *inputURI)
 
     E( readTreeFromURI(S, styleSheet, temp = sheetURI, theBase, TRUE) );
     //styleSheet -> speakDebug();
+    if ( S.hasFlag(SAB_DUMP_SHEET_STRUCTURE) )
+      styleSheet -> dumpStructure(S);
 
 #ifdef SABLOT_DEBUGGER
     if (debugger) debugger -> setBreakpoints(S, styleSheet);
@@ -360,7 +350,7 @@ eFlag Processor::run(Sit S, const char* resultURI, NodeHandle doc /*= NULL */)
     E( outputter() -> eventEndOutput(S) );
 
     //done all outputetrs
-    E( outputDocuments.finish(S) );
+    //E( outputDocuments.finish(S) );
         
     // report info about the output document to the MiscHandler
     void *miscUserData;
@@ -647,44 +637,44 @@ eFlag Processor::readTreeFromURI(Sit S, Tree*& newTree, const Str& location,
     return OK;
 }
 
-eFlag Processor::createOutputterForURI(Sit S, Str& location, Str& base,
+eFlag Processor::createOutputterForURI(Sit S, //Str& location, Str& base,
+				       Str& absolute,
 				       OutputterObj*& result,
 				       OutputDefinition *outDef /*=null*/)
 {
-  Str absolute;
-  makeAbsoluteURI(S, location, base, absolute);
+  //Str absolute;
+  //makeAbsoluteURI(S, location, base, absolute);
   if (datalines.getTree(absolute, FALSE, DLMODE_WRITE))
     Err1(S, E1_URI_WRITE, absolute);
   GP( DataLine ) d;
-  if ( !(absolute == "arg:/null") ) { 
-    M( S, d = new DataLine );
-    E( (*d).open(S, absolute, DLMODE_WRITE, &argList) );
-    // FIXME: the NULL tree in the following
-    E( datalines.addLine(S, d.keep(), NULL, FALSE) );
-  }else{ //physical output not required
-    d = NULL;
-  };
+  if ( !(absolute == "arg:/null") ) 
+    { 
+      M( S, d = new DataLine );
+      E( (*d).open(S, absolute, DLMODE_WRITE, &argList) );
+      // FIXME: the NULL tree in the following
+      E( datalines.addLine(S, d.keep(), NULL, FALSE) );
+    } else { //physical output not required
+      d = NULL;
+    };
   GP( OutputterObj ) newOut;
   M( S, newOut = new OutputterObj );
-
+  
   if (!outDef) outDef = &(styleSheet -> outputDef);
   E( (*newOut).setOptions(S, d, outDef) );
   if (theSAXHandler)
     E( (*newOut).setOptionsSAX(S, theSAXHandler, theSAXUserData, 
 			       SAXOUTPUT_AS_PHYSICAL) );
-
+  
   result = newOut.keep();
-
+  
   return OK;
 }
 
-eFlag Processor::createOutputDocument(Sit S, Str& href,
+eFlag Processor::getOutputDocument(Sit S, Str& href,
 				      OutputDocument *&resultDoc, 
 				      OutputDefinition *def)
 {
-
-  //OutputterObj *newOut;
-  //E( createOutputterForURI(S, href, base, newOut, def) );
+  //OutputDocument * aux = outputDocuments.find(def -> 
   
   //GP(OutputDocument) doc = new OutputDocument(newOut, def);
   GP(OutputDocument) doc = new OutputDocument(href, def);
@@ -700,8 +690,10 @@ eFlag Processor::startDocument(Sit S, OutputDocument *doc)
   switch (doc -> getState()) {
   case OUTDOC_ACTIVE:
     {
-      OutputterObj *out = NZ( doc -> getOutputter());
-      E( pushOutputter(S, out) );
+      assert(!"rewrite document");
+      //printf("---> aaa\n");
+      //OutputterObj *out = NZ( doc -> getOutputter());
+      //E( pushOutputter(S, out) );
     }; break;
   case OUTDOC_NEW:
     {
@@ -715,18 +707,29 @@ eFlag Processor::startDocument(Sit S, OutputDocument *doc)
 	{
 	  base = phys -> getDataLine() -> fullUri;
 	}
-      if (base == "" || base == "file://stdout" || base == "file://stderr")
+      if (base == "" || base == "file:///__stdout" || base == "file:///__stderr")
 	{
 	  DStr cwd;
 	  my_getcwd(cwd);
-	  base = "file:";
+	  base = "file://";
 	  base += cwd;
 	  S.message(MT_LOG, L2_SUBDOC_BASE, href, base);
 	}
       S.message(MT_LOG, L2_SUBDOC, href, base);
 
       OutputterObj *newOut;
-      E( createOutputterForURI(S, href, base, newOut, doc->getDefinition()) );
+      Str absolute;
+      makeAbsoluteURI(S, href, base, absolute);
+
+      //test if the document was not output recently
+      if ( outputDocumentURIs.findIdx(absolute) != -1 ) {
+	Err1( S, E_DUPLICIT_OUTDOC, absolute );
+      } else {
+	doc -> setURI(absolute);
+	outputDocumentURIs.insert(new Str(absolute));
+      }
+
+      E( createOutputterForURI(S, absolute, newOut, doc->getDefinition()) );
       E( pushOutputter(S, doc -> setOutputter(newOut)) );
       E( outputter() -> eventBeginOutput(S) );
 
@@ -746,13 +749,8 @@ eFlag Processor::finishDocument(Sit S, OutputDocument *doc, Bool canClose)
     assert(!"Could not finish unopened/finished document"); break;
   case OUTDOC_ACTIVE:
     {
-      if (canClose)
-	{
-	  E( outputter() -> eventTrailingNewline(S) );
-	  E( outputter() -> eventEndOutput(S) );
-	  doc -> setState(OUTDOC_FINISHED);
-	}
-      E( popOutputterNoFree(S) );
+      E( doc -> finish(S) );
+      E( popOutputter(S) );
     }; break;
   }
   return OK;
@@ -763,7 +761,9 @@ eFlag Processor::pushOutputterForURI(Sit S, Str& location, Str& base,
 {
   OutputterObj *newOut;
 
-  E( createOutputterForURI(S, location, base, newOut, outDef) );
+  Str absolute;
+  makeAbsoluteURI(S, location, base, absolute);
+  E( createOutputterForURI(S, absolute, newOut, outDef) );
 
   outputters_.append(newOut);
   return OK;
@@ -1011,20 +1011,20 @@ eFlag Processor::useArg(Sit S, const char *name, const char *val)
 
 eFlag Processor::useTree(Sit S, const char *name, Tree *t)
 {
-    assert(name);
-    DStr nameStr;
-    if (*name != '/')
-        nameStr = "/";
-    nameStr += name;
-	// to check for duplicate trees
-	E( useArg(S, name, NULL) );
-	
-	Str absolute;
-	DataLine *d;
-	makeAbsoluteURI(S, nameStr, "arg:", absolute);
-	E( addLineTreeOnly(S, d, absolute, t -> XSLTree, t) );
-    addedFlag = TRUE;
-    return OK;
+  assert(name);
+  DStr nameStr;
+  if (*name != '/')
+    nameStr = "/";
+  nameStr += name;
+  // to check for duplicate trees
+  E( useArg(S, name, NULL) );
+  
+  Str absolute;
+  DataLine *d;
+  makeAbsoluteURI(S, nameStr, "arg:/", absolute);
+  E( addLineTreeOnly(S, d, absolute, t -> XSLTree, t) );
+  addedFlag = TRUE;
+  return OK;
 }
 
 eFlag Processor::addGlobalParam(Sit S, const char *name, const char *val)
@@ -1445,6 +1445,7 @@ Bool Processor::isInBinding()
   return inBinding.number() && inBinding.last();
 }
 
+/*
 eFlag Processor::pushDocumentDefinition(Sit S, OutputDefinition *def,
 					OutputterObj*& out)
 {
@@ -1480,3 +1481,5 @@ eFlag Processor::popDocumentDefinition(Sit S)
     }
   return OK;
 }
+
+*/
